import os
from fpdf import FPDF
from faker import Factory
from random import randint, choice
import pandas as pd
from unidecode import unidecode
from path import Path
from warnings import simplefilter
from datetime import datetime, timedelta

simplefilter("ignore")

MEDIC = [
    "DOLIPRANE",
    "DAFALGAN",
    "IMODIUM",
    "SPASFON",
    "TAHOR",
    "SPEDIFEN",
    "VOLTARENE",
    "ELUDRIL",
    "IXPRIM",
    "PARACETAMOL BIOGARAN",
    "FORLAX",
    "MAGNE B6",
    "HELICIDINE",
    "CLAMOXYL",
    "PIASCLEDINE",
    "LAMALINE",
    "GAVISCON",
    "DAFLON",
    "ANTARENE",
    "RHINOFLUIMUCIL",
    "PLAVIX",
    "MOPRAL",
    "SUBUTEX",
    "AERIUS",
    "ORELOX",
    "INEXIUM",
    "METEOSPASMYL",
    "AUGMENTIN",
    "TOPLEXIL",
    "PIVALONE",
    "VASTAREL",
    "ADVIL",
    "EUPANTOL",
    "DEXERYL",
    "RENUTRYL 500",
    "XANAX",
    "EMLAPATCH",
    "LASILIX",
    "ENDOTELON",
    "DEROXAT",
    "TEMESTA",
    "EFFEXOR",
    "PARACETAMOL SANDOZ",
    "VENTOLINE",
    "SOLUPRED",
    "DEXTROPROPOXYPHENE PARAC BIOG",
    "PNEUMOREL",
    "INIPOMP",
    "PREVISCAN",
    "ASPEGIC",
    "GINKOR",
    "CRESTOR",
    "MEDIATOR",
    "SERESTA",
    "MOTILIUM",
    "PARACETAMOL MERCK",
    "CELESTENE",
    "AMLOR",
    "DIAMICRON",
    "TANAKAN",
    "ATARAX",
    "DERINOX",
    "XYZALL",
    "DEXTROPROPOXYPHENE PARAC SAND",
    "SERETIDE",
    "COVERSYL",
    "PROPOFAN",
    "HEXAQUINE",
    "APROVEL",
    "PARIET",
    "ZALDIAR",
    "DIPROSONE",
    "PARACETAMOL TEVA",
    "BETADINE",
    "LYSANXIA",
    "ALODONT",
    "LEXOMIL",
    "DACRYOSERUM",
    "FUCIDINE",
    "STILNOX",
    "KETUM",
    "STABLON",
    "ART",
    "BIOCALYPTOL",
    "THIOVALONE",
    "DEBRIDAT",
    "PYOSTACINE",
    "TIORFAN",
    "SPECIAFOLDINE",
    "OGAST",
    "RIVOTRIL",
    "TOPALGIC",
    "NASONEX",
]

POSO_TYPE = ["cp", "injections", "comprimés"]

POSO_FREQ = [
    "le matin",
    "le soir",
    "matin, midi et soir",
    "avant les repas",
    "après les repas",
    "par jour",
]

POSO_DURATION = [f"pendant {n} jours" for n in {3, 5, 7, 10, 15, 30}] + ["si douleur"]

MONTHS = [
    "janvier",
    "février",
    "mars",
    "avril",
    "mai",
    "juin",
    "août",
    "septembre",
    "octobre",
    "novembre",
    "décembre",
]

PATHOLOGY = ["fièvre",
             "toux",
             "radiographie pulmonaire",
             "procédure TAVI",
             "remplacement valvulaire",
             "fracture du col du fémur"]

ANT = ["fumeur",
       "hypertension artérielle",
       "hypercholestérolélie",
       "diabète",
       "diabète de type 2"
       "obésité"]

FONTS_FOLDER_PATH = Path(os.path.dirname(__file__)) / "utils/fonts/"

SIGNATURE_FONTS = [
    FONTS_FOLDER_PATH / name
    for name in os.listdir(FONTS_FOLDER_PATH)
    if name.endswith(".ttf")
]

MAIN_FONTS = ["Arial", "Courier", "Helvetica", "Times"]

fake = Factory.create("fr_FR")


def add_block(pdf, txt, align="L"):
    """Adds paragraph to the pdf object

    Arguments:
        pdf {fpdf.fpdf.FPDF} -- current pdf object
        txt {str} -- paragraph to add

    Keyword Arguments:
        align {str} -- text alignment : the possible values are "L", "C" and "R" (default: {"L"})
    """

    for row in txt.split("\n"):
        pdf.cell(200, 5, txt=row, ln=1, align=align)


def signature(name):
    """Creates a fake signature

    Arguments:
        name {str} -- person name

    Returns:
        str -- signature
    """

    name_list = unidecode(name).split(" ")
    sign = ""
    for w in name_list[:-1]:
        if randint(0, 1):
            if randint(0, 1):
                sign += w[0]
            else:
                sign += w
            if randint(0, 1):
                sign += " "
    sign += name_list[-1]
    return sign


def prescription_generator(output_path, patient_name, year):
    """Generates a fake medical prescription

    Arguments:
        output_path {Path} -- path the ouput pdf file
        patient_name {str} -- patient name to use
        year {int} -- year of the prescription
    """

    pdf = FPDF(format="A4")
    pdf.add_page()
    font = choice(MAIN_FONTS)
    pdf.set_font(font, size=11)

    # En-tête

    txt = f"""À {fake.city()}, le {randint(2, 28)} {choice(MONTHS)} {year}     
    
    
    
    """
    add_block(pdf, txt, align="R")

    # Centre médical

    city = fake.city()
    postal_code = randint(1000, 10000) * 10

    txt = f"""
Centre Médical de {city}
{fake.street_address()}
{postal_code} {city}
{fake.phone_number()}


"""
    pdf.set_font(font, "B", size=11)
    add_block(pdf, txt, align="C")
    pdf.set_font(font, size=11)

    # Docteur

    doctor_name = fake.name()
    postal_code = randint(1000, 10000) * 10

    txt = f"""
        Docteur {doctor_name}
        {fake.street_address()}
        {postal_code} {fake.city()}
        Tél. : {fake.phone_number()}

"""
    add_block(pdf, txt)

    # Patient

    txt = f"""
        {patient_name}
        {randint(12, 100)} ans, {randint(30, 110)} kg

    """

    add_block(pdf, txt)

    # Prescription

    nb_medics = randint(1, 5)
    for _ in range(nb_medics):

        txt = f"""
        {choice(MEDIC)} {5 * randint(1, 10)} mg"""

        pdf.set_font(font, "B", size=11)
        add_block(pdf, txt)

        txt = f"""          {randint(1, 3)} {choice(POSO_TYPE)} {choice(POSO_FREQ)} {choice(POSO_DURATION)}
"""

        pdf.set_font(font, size=11)
        add_block(pdf, txt)

    # Signature

    pdf.add_font("Signature", "", choice(SIGNATURE_FONTS), uni=True)
    font_size = randint(20, 40)
    pdf.set_font("Signature", size=font_size)

    txt = f"""


{signature(doctor_name)}          {int((40 - font_size)/20) * randint(0, 20) * ' '}
"""
    add_block(pdf, txt, align="R")

    pdf.output(output_path)

def cr_generator(output_path, patient_name, year):
    """Generates a fake medical report

    Arguments:
        output_path {Path} -- path the ouput pdf file
        patient_name {str} -- patient name to use
        year {int} -- year of the prescription
    """
    pdf = FPDF(format="A4")
    pdf.add_page()
    font = choice(MAIN_FONTS)

    # Centre médical

    city = fake.city()
    postal_code = randint(1000, 10000) * 10

    txt = f"""
Centre Médical de {city}
{fake.street_address()}
{postal_code} {city}
{fake.phone_number()}


"""
    pdf.set_font(font, "B", size=11)
    add_block(pdf, txt, align="L")
    pdf.set_font(font, size=11)

    # Docteur

    doctor_name = fake.name()
    postal_code = randint(1000, 10000) * 10

    txt = f"""
Docteur {doctor_name}
{fake.street_address()}
{postal_code} {fake.city()}
Tél. : {fake.phone_number()}

"""
    add_block(pdf, txt)

    # Date
    last_date = fake.date_between(start_date=datetime(int(year), 1, 1), end_date=datetime(int(year), 12, 12))
    print(last_date)

    txt = f"""À {fake.city()}, le {last_date.strftime("%d/%m/%Y")}     

        """
    add_block(pdf, txt, align="R")

    #Objet
    duration = randint(1,16)
    first_date = last_date - timedelta(days=duration)

    txt = f"""Objet : Compte-rendu d'hospitalisation de {patient_name} du {first_date.strftime("%d/%m/%Y")} au {last_date.strftime("%d/%m/%Y")}

    {patient_name} né le {fake.date_between(start_date='-80y', end_date=first_date).strftime("%d/%m/%Y")} a été hospitalisé le {first_date.strftime("%d/%m/%Y")} pour {choice(PATHOLOGY)}

"""
    add_block(pdf, txt, align="L")

    #Observation
    pdf.set_font(font, "B", size=11)
    txt = """
    Observation
    """
    add_block(pdf, txt, align="L")
    pdf.set_font(font, size=11)

    #Antécédents
    pdf.set_font(font, "B", size=11)
    txt = """
    Antécédents
    """
    add_block(pdf, txt, align="L")
    pdf.set_font(font, size=11)
    nb_ant = randint(1, 3)
    for _ in range(nb_ant):
        txt = f"""  {choice(ANT)} """
        add_block(pdf, txt)


    #Traitement de sortie
    pdf.set_font(font, "B", size=11)
    txt = """
    Traitement de sortie
    """
    add_block(pdf, txt, align="L")
    pdf.set_font(font, size=11)
    nb_medics = randint(1, 4)
    for _ in range(nb_medics):

        txt = f"""
        {choice(MEDIC)} {5 * randint(1, 10)} mg, {randint(1, 3)} {choice(POSO_TYPE)} {choice(POSO_FREQ)} {choice(POSO_DURATION)} 
    """
        add_block(pdf, txt)

    # Signature

    pdf.add_font("Signature", "", choice(SIGNATURE_FONTS), uni=True)
    font_size = randint(20, 40)
    pdf.set_font("Signature", size=font_size)

    txt = f"""


{signature(doctor_name)}          {int((40 - font_size) / 20) * randint(0, 20) * ' '}
"""
    add_block(pdf, txt, align="R")

    pdf.output(output_path)

def get_id():
    """Generates a random 6-digit id

    Returns:
        str -- random 6-digit id
    """

    return str(randint(1000, 100000)).zfill(6)


def generate(nb, output_path):
    """Generate several fake medical prescriptions

    Arguments:
        nb {int} -- number or fake medical prescriptions to generate
        output_path {str} -- folder to store the fake medical prescriptions in
    """

    output_path = Path(output_path)

    link = pd.DataFrame(
        columns=[
            "patient_id",
            "patient_surname",
            "patient_name",
            "date_folder",
            "prescription_id",
            "cr_id"
        ]
    )

    print("Generating data...")

    for i in range(nb):

        print(f"Loading : {int(100 * i / nb)} %", end="\r")

        patient_id = get_id()
        while not link[lambda row: row["patient_id"] == patient_id].empty:
            patient_id = get_id()

        prescription_id = get_id()
        while not link[lambda row: row["prescription_id"] == prescription_id].empty:
            prescription_id = get_id()

        cr_id = get_id()
        while not link[lambda row: row["prescription_id"] == prescription_id].empty:
            cr_id = get_id()

        patient_name = fake.first_name()
        patient_surname = fake.last_name()
        year = str(randint(2010, 2019))

        link = link.append(
            {
                "patient_id": patient_id,
                "patient_surname": patient_surname,
                "patient_name": patient_name,
                "date_folder": year,
                "prescription_id": prescription_id,
                "cr_id": cr_id,
            },
            ignore_index=True,
        )

        if not os.path.isdir(output_path / year):
            os.mkdir(output_path / year)

        prescription_generator(
            f"{output_path}/{year}/prescription_{prescription_id}.pdf",
            patient_name + " " + patient_surname,
            year,
        )
        cr_generator(
            f"{output_path}/{year}/CR_{cr_id}.pdf",
            patient_name + " " + patient_surname,
            year,
        )

    link.to_csv(output_path / "index.csv")

    print("Generation terminated !")



from faker import Factory
from random import choice, choices, randint, random, shuffle


class DataBaseRandomGenerator:
    def __init__(self, locale="fr_FR"):
        self._fake_factory = Factory.create(locale)
        self.locale = locale
        self.tables = {}
        self._possible_values = {}
        self.relations = {}

    def add_table(self, name, table):
        assert not name in self.tables, f"The table {name} already exists."
        self.tables[name] = table
        self._possible_values[name] = {}

    def add_relation(self, name, relation_dict):
        assert (
            type(relation_dict) == dict
            and "parents" in relation_dict.keys()
            and "children" in relation_dict.keys()
        ), "A relation must be a dict following the pattern {'parents': (table, attribute), 'children': (table, attribute, type)}."
        test_table_name, test_attr_name = relation_dict["parents"][0]
        attribute_type = (
            self.tables[test_table_name].attributes[test_attr_name].attr_type.type_name
        )
        assert name not in self.relations, f"The relation {name} already exists."
        assert (
            test_table_name in self.tables.keys()
        ), f"A relation uses the table {test_table_name} whereas this table does not exist."
        for table_name, attr_name in relation_dict["parents"]:
            assert (
                table_name in self.tables.keys()
            ), f"A relation uses the table {table_name} whereas this table does not exist."
            assert (
                attr_name in self.tables[table_name].attributes.keys()
            ), f"A relation uses the attribute {attr_name} from table {table_name} whereas this attribute does not exist."
            assert (
                self.tables[table_name].attributes[attr_name].attr_type.type_name
                == attribute_type
            ), f"The following attributes are in relation but their types are different : {attr_name} from {table_name} (of type {self.tables[table_name].attributes[attr_name].attr_type.type_name}) and {test_attr_name} from {test_table_name} (of type {attribute_type})."
        for table_name, attr_name, _ in relation_dict["children"]:
            assert (
                table_name in self.tables.keys()
            ), f"A relation uses the table {table_name} whereas this table does not exist."
            assert (
                attr_name in self.tables[table_name].attributes.keys()
            ), f"A relation uses the attribute {attr_name} from table {table_name} whereas this attribute does not exist."
            assert (
                self.tables[table_name].attributes[attr_name].attr_type.type_name
                == attribute_type
            ), f"The following attributes are in relation but their types are different : {attr_name} from {table_name} (of type {self.tables[table_name].attributes[attr_name].attr_type.type_name}) and {test_attr_name} from {test_table_name} (of type {attribute_type})."
        self.relations[name] = relation_dict

    def _satisfy_relation(
        self, relation_dict, min_rows=10, max_card=5, missing_rate=0.1
    ):
        test_table_name, test_attr_name = relation_dict["parents"][0]
        length = max(
            [
                self.tables[table_name].length()
                for table_name, _ in relation_dict["parents"]
            ]
        )
        attribute_class = self.tables[test_table_name].attributes[test_attr_name]
        for table_name, attr_name in relation_dict["parents"]:
            assert self.tables[table_name].length() in {
                0,
                length,
            }, f"The tables {test_table_name} and {table_name} are in relation one-to-one with the attributes {test_attr_name} = {attr_name} but their length are different : {length} != {self.tables[table_name].length}."
        if not length:
            length = min_rows
        possible_values = []
        if attribute_class.attr_type.type_name == "field":
            possible_values = attribute_class.kwargs_rd_gen["possible_values"].to_list()
        else:
            for _ in range(length):
                val = attribute_class.get_random_value(self._fake_factory, self.locale)
                while val in possible_values:
                    val = attribute_class.get_random_value(
                        self._fake_factory, self.locale
                    )
                possible_values.append(val)
        length_possible_values = len(possible_values)
        for table_name, attr_name in relation_dict["parents"]:
            self._possible_values[table_name][attr_name] = possible_values
        for table_name, attr_name, rel_type in relation_dict["children"]:
            assert rel_type != (2, 2), "Cannot handle (2,2) relations yet."
            if rel_type == (0, 1):
                length = self.tables[table_name].length()
                if length == 0:
                    self._possible_values[table_name][attr_name] = []
                    for val in possible_values:
                        if random() > missing_rate:
                            self._possible_values[table_name][attr_name].append(val)
                else:
                    assert (
                        length <= length_possible_values
                    ), f"Not enough values to fit in attribute {attr_name} from {table_name}."
                    self._possible_values[table_name][attr_name] = choices(
                        possible_values, k=length
                    )
            elif rel_type == (0, 2):
                length = self.tables[table_name].length()
                shuffle(possible_values)
                current_possible_values = possible_values[
                    : int(length_possible_values * (1 - missing_rate))
                ]
                if length == 0:
                    self._possible_values[table_name][attr_name] = choices(
                        current_possible_values, k=int(min_rows * max_card / 2)
                    )
                else:
                    assert length >= len(
                        possible_values
                    ), f"Too many values to fit in attribute {attr_name} from table {table_name}."
                    self._possible_values[table_name][attr_name] = choices(
                        current_possible_values, k=length
                    )
            elif rel_type == (1, 2):
                length = self.tables[table_name].length()
                if length == 0:
                    current_possible_values = possible_values + choices(
                        possible_values, k=int(min_rows * max_card / 2) - 1
                    )
                    shuffle(current_possible_values)
                    self._possible_values[table_name][
                        attr_name
                    ] = current_possible_values
                else:
                    assert length >= len(
                        possible_values
                    ), f"Too many values to fit in attribute {attr_name} from table {table_name}."
                    current_possible_values = possible_values + choices(
                        possible_values, k=length - length_possible_values
                    )
                    shuffle(current_possible_values)
                    self._possible_values[table_name][
                        attr_name
                    ] = current_possible_values

    def random_generation(self, min_rows=100, max_card=5, missing_rate=0.1):

        print("Starts filling table...")
        nb_relations = len(self.relations)
        r = 0
        for _, relation_dict in self.relations.items():
            print(
                f"Satisfying relations : {int(50 * (r + 1) / nb_relations) * '■' + int(50 * (1 - (r + 1) / nb_relations)) * '_'}",
                end="\r",
            )
            self._satisfy_relation(
                relation_dict,
                min_rows=min_rows,
                max_card=max_card,
                missing_rate=missing_rate,
            )
            r += 1
        print("\n")
        nb_tables = len(self.tables)
        t = 0
        for table in self.tables:
            print(
                f"Filling columns      : {int(50 * (t + 1) / nb_tables) * '■' + int(50 * (1 - (t + 1) / nb_tables)) * '_'}",
                end="\r",
            )
            for i in range(
                min(
                    [min_rows]
                    + [
                        len(self._possible_values[table][attr])
                        for attr in self._possible_values[table].keys()
                    ]
                )
            ):
                self.tables[table].add_random_row(
                    self._fake_factory,
                    self.locale,
                    fixed_values={
                        attr: values[i]
                        for attr, values in self._possible_values[table].items()
                    },
                )
            t += 1
        print("\nDatabase filled !")



import csv
import os
import pandas as pd
import re

from selenium import webdriver
from selenium.webdriver.firefox.options import Options

from warnings import simplefilter

simplefilter("ignore")


class MimicPreprocessing:
    def __init__(self, dest_folder="data"):
        self.BASE_LINK = "https://alpha.physionet.org/files/mimiciii-demo/1.4/"
        self.dest_folder = dest_folder
        if not os.path.isdir(dest_folder):
            os.mkdir(dest_folder)
            os.mkdir(f"{dest_folder}/csv")
            os.mkdir(f"{dest_folder}/csv_unique_values")

    def scraper(self, unused_tables=[]):

        print("Starts scraping mimic...")

        options = Options()
        options.headless = True
        browser = webdriver.Firefox(options=options, log_path="/tmp/test")
        browser.get(self.BASE_LINK)

        filename_list = [x.text for x in browser.find_elements_by_xpath("//a[@href]")]

        i = 0
        for filename in filename_list:

            if not filename.split(".csv")[0].lower() in unused_tables:

                progress = (i + 1) / len(filename_list)
                print(
                    str("\r" + int(100 * progress) * "■" + int(100 * (1 - progress)) * "_"), end="",
                )

                if filename.endswith(".csv"):

                    browser.get(self.BASE_LINK + filename)
                    txt = browser.find_element_by_tag_name("pre").text

                    with open(f"{self.dest_folder}/csv/{filename.lower()}", "w+") as f:
                        f.write(txt)

                i += 1

        print("\nScrapping terminated !\n")

    def infer_attributes(self):

        print("Starts inferring attributes from mimic...")

        attribute_types_all_tables = {}

        possible_values_all_tables = {}

        targets = os.listdir(f"{self.dest_folder}/csv")

        f_count = 0

        for filename in targets:

            if filename.endswith(".csv"):
                progress = (f_count + 1) / len(targets)
                print(
                    int(100 * progress) * "■" + int(100 * (1 - progress)) * "_", end="\r",
                )

                attribute_types = {}
                possible_values = {}
                df = pd.read_csv(f"{self.dest_folder}/csv/{filename}").applymap(str)
                unique_values = {}
                attr_list = df.columns
                for attr in attr_list:
                    unique_values[attr] = df[attr].drop_duplicates().reset_index(drop=True)
                    try:
                        attr_values = unique_values[attr]
                        i = 0
                        while attr_values[i] == "":
                            i += 1
                        if (
                            False
                        ):  ############################################ TO FIX WHEN ATTRIBUTE TYPE DETECTION HAS BEEN SUFFICIENTLY IMPROVED (bad guessed type may lead to troubles with relations)
                            if re.match(r"^\d+$", attr_values[i]):
                                float_type = False
                                ev_digit_id_type = False
                                relative_float_type = False
                                length = len(attr_values[i])
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if not re.match(r"^\d+$", attr_values[j]):
                                            if re.match(r"^\d+\.\d+$", attr_values[j]):
                                                float_type = True
                                            elif re.match(r"^-\d+\.?\d*$", attr_values[j]):
                                                relative_float_type = True
                                            elif re.match(r"^[EV]?\d+$", attr_values[j]):
                                                ev_digit_id_type = True
                                            else:
                                                attribute_types[attr] = f"field-{attr}"
                                                possible_values[f"{attr}"] = attr_values
                                                is_field = True
                                                break
                                        else:
                                            length = max(length, len(attr_values[j]))
                                if not is_field:
                                    if ev_digit_id_type:
                                        attribute_types[attr] = f"ev_digit_id-"
                                    elif relative_float_type:
                                        min_value = min(
                                            [float(x) for x in attr_values if float(x) > 0]
                                        )
                                        max_value = max(
                                            [float(x) for x in attr_values if float(x) > 0]
                                        )
                                        attribute_types[
                                            attr
                                        ] = f"relative_float-{min_value}-{max_value}"
                                    elif float_type:
                                        min_value = min([float(x) for x in attr_values])
                                        max_value = max([float(x) for x in attr_values])
                                        attribute_types[attr] = f"float-{min_value}-{max_value}"
                                    else:
                                        attribute_types[attr] = f"digit_id-{length}"
                            elif (
                                re.match(r"^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$", attr_values[i],)
                                or attr_values[i] == "nan"
                            ):
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if (
                                            not re.match(
                                                r"^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$",
                                                attr_values[j],
                                            )
                                            or attr_values[i] == "nan"
                                        ):
                                            attribute_types[attr] = f"field-{attr}"
                                            possible_values[f"{attr}"] = attr_values
                                            is_field = True
                                            break
                                if not is_field:
                                    attribute_types[attr] = f"date-"
                            elif re.match(r"^\d+\.\d+$", attr_values[i]) or attr_values[i] == "nan":
                                relative_float_type = False
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if (
                                            not re.match(r"^\d+\.?\d*$", attr_values[j])
                                            or attr_values[j] == "nan"
                                        ):
                                            if re.match(r"^-\d+\.?\d*$", attr_values[j]):
                                                relative_float_type = True
                                            else:
                                                attribute_types[attr] = f"field-{attr}"
                                                possible_values[f"{attr}"] = attr_values
                                                is_field = True
                                                break
                                if not is_field:
                                    if relative_float_type:
                                        min_value = min(
                                            [float(x) for x in attr_values if float(x) > 0]
                                        )
                                        max_value = max(
                                            [float(x) for x in attr_values if float(x) > 0]
                                        )
                                        attribute_types[
                                            attr
                                        ] = f"relative_float-{min_value}-{max_value}"
                                    else:
                                        min_value = min([float(x) for x in attr_values])
                                        max_value = max([float(x) for x in attr_values])
                                        attribute_types[attr] = f"float-{min_value}-{max_value}"
                            elif (
                                re.match(r"^-\d+\.?\d*$", attr_values[i]) or attr_values[i] == "nan"
                            ):
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if (
                                            not re.match(r"^-\d+\.?\d*$", attr_values[j])
                                            or attr_values[j] == "nan"
                                        ):
                                            attribute_types[attr] = f"field-{attr}"
                                            possible_values[
                                                f"{filename.split('.')[0]}-{attr}"
                                            ] = attr_values
                                            is_field = True
                                            break
                                if not is_field:
                                    min_value = min(
                                        [abs(float(x)) for x in attr_values if float(x) > 0]
                                    )
                                    max_value = max([float(x) for x in attr_values if float(x) > 0])
                                    attribute_types[
                                        attr
                                    ] = f"relative_float-{min_value}-{max_value}"
                            elif re.match(r"^\d{5}-\d{5}$", attr_values[i]):
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if not re.match(r"^\d{5}-\d{5}$", attr_values[j]):
                                            attribute_types[attr] = f"field-{attr}"
                                            possible_values[f"{attr}"] = attr_values
                                            is_field = True
                                            break
                                if not is_field:
                                    attribute_types[attr] = f"double_digit_id-5"
                            elif re.match(r"^\d+-\d$", attr_values[i]) or attr_values[i] == "nan":
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if (
                                            not re.match(r"^\d+-\d$", attr_values[j])
                                            or attr_values[j] == "nan"
                                        ):
                                            attribute_types[attr] = f"field-{attr}"
                                            possible_values[f"{attr}"] = attr_values
                                            is_field = True
                                            break
                                if not is_field:
                                    attribute_types[attr] = f"composed_digit_id-"
                            elif re.match(r"^[EV]?\d+$", attr_values[i]):
                                is_field = False
                                for j in range(i + 1, len(attr_values)):
                                    if attr_values[j] != "":
                                        if not re.match(r"^[EV]?\d+$", attr_values[j]):
                                            attribute_types[attr] = f"field-{attr}"
                                            possible_values[f"{attr}"] = attr_values
                                            is_field = True
                                            break
                                if not is_field:
                                    attribute_types[attr] = f"ev_digit_id-"
                        else:
                            attribute_types[attr] = f"field-{attr}"
                            possible_values[f"{attr}"] = attr_values
                    except IndexError:
                        print(f"Attributed ignored : empty column {attr} in table {filename}.")

                with open(f"{self.dest_folder}/csv_unique_values/{filename}", "w+") as f:
                    writer = csv.writer(f)
                    writer.writerow(unique_values.keys())
                    max_len = max(len(unique_values[x]) for x in attr_list)
                    i = 0
                    while i < max_len:
                        row = []
                        for attr in attr_list:
                            values = unique_values[attr]
                            if i < len(values):
                                row.append(values[i])
                            else:
                                row.append("")
                        writer.writerow(row)
                        i += 1
                attribute_types_all_tables[filename.split(".")[0].lower()] = attribute_types
                possible_values_all_tables[filename.split(".")[0].lower()] = possible_values
                f_count += 1

        print("\nInferring terminated !\n")
        return attribute_types_all_tables, possible_values_all_tables



from os import path
from setuptools import setup, find_packages

with open(path.join(path.abspath(path.dirname(__file__)), "README.md"), encoding="utf-8",) as f:
    long_description = f.read()

setup(
    name="pysin",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        "console_scripts": ["pysin-gen=mimic.cli:generate", "pysin-scrap=mimic.cli:scrap",]
    },
    version="1.2.0",
    license="Apache License 2.0",
    description="PySin is a toolbox for text retrieval in unstructured documents datasets. It contains both a multi-type text extractor and a search engine. To test them, you can use the medical prescriptions generator that is also provided.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Jean-Baptiste Laval",
    author_email="contact@arkhn.com",
    url="https://github.com/arkhn/PySin",
    download_url="https://github.com/arkhn/PySin/archive/1.1.0.tar.gz",
    keywords=[
        "arkhn",
        "text retrieval",
        "search engine",
        "text extraction",
        "dataset generator",
        "medical",
    ],
    install_requires=[
        "pandas==1.0.3",
        "fpdf==1.7.2",
        "Faker==4.1.0",
        "Unidecode==1.1.1",
        "docx2txt==0.8",
        "path.py==12.4.0",
        "pdftotext==2.1.4",
        "striprtf==0.0.10",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
